@@ -11,12 +11,12 @@ import functools
 import uuid
 import zlib
 
-from socket import error as socket_error, AF_INET
-import socket as _socket
-import gevent
-from gevent.socket import EBADF, EWOULDBLOCK, socket, create_connection
-from gevent.server import StreamServer
+from socket import error as socket_error, EBADF, AF_INET, SOL_SOCKET, SO_REUSEADDR
+
+from gevent.socket import socket, create_connection
 from gevent.coros import Semaphore
+from gevent import GreenletExit, spawn, Timeout, sleep
+from gevent.event import AsyncResult
 
 #from pickle import loads as pickle_loads, dumps as pickle_dumps, HIGHEST_PROTOCOL
 from cPickle import loads as pickle_loads, dumps as pickle_dumps, HIGHEST_PROTOCOL
@@ -29,10 +29,6 @@ try:
 except ImportError:
     from cPickle import loads, dumps, load, dump
 
-from gevent import GreenletExit, spawn, spawn_later, Timeout, sleep
-from gevent.pool import Pool
-from gevent.queue import Queue
-from gevent.event import AsyncResult
 
 from logging import root as logger, currentframe
 
@@ -75,13 +71,6 @@ class FakeTimeout:
 fake_timeout = FakeTimeout()
 timeout_error = RpcTimeout()
 
-_context = None
-def get_context():
-    global _context
-    if _context is None:
-        _context = Context(3)
-    return _context
-
 #rpc数据类型
 RT_REQUEST = 1 << 0
 RT_RESPONSE = 1 << 1
@@ -105,14 +94,17 @@ HEARTBEAT_TIME = 60 #heartbeat, if disconnect time > (HEARTBEAT_TIME + RECONNECT
 CALL_TIMEORUT = 240
 ZIP_LENGTH = 1024 * 2 #if argkw > Nk, use zlib to compress
 ZIP_LEVEL = 3
+MAX_INDEX = (1 << 32) - 1
 
 use_logging = False
 log_head = '[p%s]' % os.getpid()
 def printf(msg, *args):
+    if args:
+        msg = msg % args
     if use_logging:
-        logger.warn(msg, *args)
+        logger.warn(msg)
     else:
-        print('%s%s' % (log_head, msg % args))
+        print('%s%s' % (log_head, msg))
 
 
 def log_except(*args, **kw):
@@ -151,14 +143,14 @@ def link(src, dest):
         if ex.errno != ENOENT:
             raise
 
-def tcp_listener(address, backlog=50, reuse_addr=True, family=_socket.AF_INET):
+def tcp_listener(address, backlog=50, reuse_addr=True, family=AF_INET):
     """A shortcut to create a TCP socket, bind it and put it into listening state."""
     sock = socket(family=family)
     if reuse_addr is not None:
-        sock.setsockopt(_socket.SOL_SOCKET, _socket.SO_REUSEADDR, reuse_addr)
+        sock.setsockopt(SOL_SOCKET, SO_REUSEADDR, reuse_addr)
     try:
         sock.bind(address)
-    except _socket.error:
+    except socket_error:
         ex = sys.exc_info()[1]
         strerror = getattr(ex, 'strerror', None)
         if strerror is not None:
@@ -265,7 +257,7 @@ class RpcBase(object):
         else:
             self._socket = create_unix_connection(address)
 
-    def close(self):
+    def _close(self):
         if not self._socket:
             return
         try:
@@ -296,7 +288,7 @@ class RpcBase(object):
         try:
             self.before_stop()
         finally:
-            self.close()
+            self._close()
 
     def _handle(self, parts):
         """ 单个数据包处理 """
@@ -307,8 +299,9 @@ class RpcBase(object):
         """ 代理obj,可以通过pickle传递到远程 """
         self.register(obj)
 
-    def register(self, export):
-        name = getattr(export, '_rpc_name_', None)
+    def register(self, export, name=None):
+        if name is None:
+            name = getattr(export, '_rpc_name_', None)
         if name:
             if name in self._exports and export != self._exports[name]():
                 raise ValueError, 'export name(%s) existed!' % name
@@ -356,7 +349,7 @@ class RpcService(object):
         self._slock = Semaphore()
         self._reconnected = None
         self.reconnect_timeout = RECONNECT_TIMEOUT
-        self.iter_id = itertools.cycle(xrange(sys.maxint))
+        self.iter_id = itertools.cycle(xrange(MAX_INDEX))
         self._resps = {}
         self._proxys = WeakValueDictionary()
         self.stoped = True
@@ -399,7 +392,7 @@ class RpcService(object):
         if 1 and not self.sock_error:
             try:
                 #printf('remote_stop:%s', self.sock_addr)
-                self.call(0, 'remote_stop', tuple(), None, no_result=True)
+                self.call('', 'remote_stop', tuple(), None, no_result=True)
                 sleep(0.05)
             except:
                 pass
@@ -504,10 +497,15 @@ class RpcService(object):
 
     def _wait_reconnect(self):
         _reconnected = self._reconnected
-        rs = _reconnected.get(timeout=self.reconnect_timeout)
+        try:
+            _reconnected.get(timeout=self.reconnect_timeout)
+        except Timeout:
+            pass
         if not _reconnected.successful():
-            self.sock_error = True
             self.stop()
+            if self.sock_error or _reconnected.exception is None:
+                return
+            self.sock_error = True
             raise _reconnected.exception
 
     def reconnect(self, sock):
@@ -552,20 +550,25 @@ class RpcService(object):
             timeout=CALL_TIMEORUT, pickle=False, proxy=False):
         dtype = RT_REQUEST
         if proxy:
-            pickle = True
             def _reg_obj(obj):
+                if hasattr(obj, 'proxy_pack'):
+                    return obj.proxy_pack(), False
                 if isinstance(obj, RpcProxy):
-                    return obj
+                    return obj._id, False
                 if hasattr(obj, '_rpc_proxy_'):
-                    return obj._rpc_proxy_()
-                return self.svr.register(obj)
-            objs = args[0]
+                    return obj._rpc_proxy_(), True
+                return self.svr.register(obj), False
+            objs = args[0] #first arg is proxy(str, RpcProxy or list)
             if isinstance(objs, (tuple, list)):
                 obj_ids = []
                 for o in objs:
-                    obj_ids.append(_reg_obj(o))
+                    obj, is_pickle = _reg_obj(o)
+                    pickle = pickle or is_pickle
+                    obj_ids.append(obj)
             else:
-                obj_ids = _reg_obj(objs)
+                obj, is_pickle = _reg_obj(objs)
+                pickle = pickle or is_pickle
+                obj_ids = obj
             args = (obj_ids, ) + args[1:]
             dtype |= DT_PROXY
         if pickle:
@@ -605,6 +608,9 @@ class RpcService(object):
                 def _get_proxy(export):
                     if isinstance(export, RpcProxy):
                         return export
+                    if isinstance(export, (tuple, list)):
+                        export_cls = PROXYS[export[0]]
+                        return export_cls.proxy_unpack(export, svc=self)
                     return self.get_proxy(export)
                 export_ids = args[0]
                 if isinstance(export_ids, (tuple, list)):
@@ -613,8 +619,12 @@ class RpcService(object):
                         proxys.append(_get_proxy(e))
                 else:
                     proxys = _get_proxy(export_ids)
-                args = (proxys, ) + args[1:]
-            rs = func(*args, **kw) if kw is not None else func(*args)
+                args = (proxys, ) + tuple(args[1:])
+
+            if args is None:
+                rs = func()
+            else:
+                rs = func(*args, **kw) if kw is not None else func(*args)
             if dtype & ST_NO_RESULT:
                 return
             if not getattr(func, '_rpc_pickle_result_', False):
@@ -625,7 +635,8 @@ class RpcService(object):
             log_except('export(%s).%s(%s)', obj_id, name, repr(argkw))
             if dtype & ST_NO_RESULT or self.svr.stoped:
                 return
-            self.send(RT_EXCEPTION, index, pickle_dumps(e, PICKLE_PROTOCOL))
+            self.send(RT_EXCEPTION, index, str(e))
+            #self.send(RT_EXCEPTION, index, pickle_dumps(e, PICKLE_PROTOCOL))
 
     def _handle_response(self, parts):
         dtype, index, argkw = parts
@@ -641,7 +652,10 @@ class RpcService(object):
 
     def _handle_exception(self, parts):
         RT_EXCEPTION, index, error = parts
-        error = pickle_loads(error)
+        #try:
+        #    error = pickle_loads(error)
+        #except:
+        error = RpcCallError(str(error))
         try:
             rs = self._resps.pop(index)
             rs.set_exception(error)
@@ -660,7 +674,7 @@ class RpcService(object):
         elif rt == RT_HEARTBEAT:
             self._heart_time = time.time()
         else:
-            raise ValueError, 'unknown data:%s' % msg
+            raise ValueError('unknown data:%s' % str(rt))
 
     def heartbeat(self):
         beat = RT_HEARTBEAT
@@ -688,7 +702,7 @@ class RpcService(object):
     #######remote call##############
     def get_export(self, export_id):
         """ get export obj by export_name """
-        if export_id == 0:
+        if not export_id:
             return self
         return self.svr.get_export(export_id)
 
@@ -731,12 +745,12 @@ class RpcService(object):
         return shell_id
 
     def stop_console(self, shell_id):
-        self.call(0, 'stop_shell', (shell_id, ), None, no_result=True, timeout=20)
+        self.call('', 'stop_shell', (shell_id, ), None, no_result=True, timeout=20)
 
     def start_console(self, pre_prompt='', shell=None):
         from rpc_shell import RpcLocalConsole, RpcProxyConsole
         console = RpcLocalConsole(self) if shell is None else RpcProxyConsole(self, shell)
-        shell_id = self.call(0, 'start_shell', (console, pre_prompt), None, proxy=True, timeout=20)
+        shell_id = self.call('', 'start_shell', (console, pre_prompt), None, proxy=True, timeout=20)
         try:
             console.wait(shell_id)
         finally:
@@ -790,7 +804,7 @@ class RpcClient(RpcBase):
             raise ValueError
 
     def execute(self, func, args, kw, **others):
-        return self.svc.call(0, 'execute', (func, args, kw),
+        return self.svc.call('', 'execute', (func, args, kw),
                 pickle=True, **others)
 
 class AccessCtrl(object):
@@ -852,7 +866,7 @@ class RpcServer(RpcBase):
                         client_socket.close()
                         printf('[rpc]access deny:%s', address)
                         return
-                    printf('[rpc]access:%s', address)
+                    #printf('[rpc]access:%s', address)
 
                     svc = self.init_service(client_socket)
                     if svc:
@@ -924,7 +938,7 @@ class RpcProxy(object):
 
     def valid(self):
         """ check this proxy is valid """
-        return self._svc.call(0, 'valid_proxy', (self._id,), None)
+        return self._svc.call('', 'valid_proxy', (self._id,), None)
 
     def get_service(self):
         return self._svc
@@ -987,6 +1001,15 @@ class DictItemProxy(RpcProxy):
         self.is_local = False
         self.export = None
 
+    def proxy_pack(self):
+        return PROXY_DICT_ITEM, self._id, self.dict_name, self.key
+
+    @classmethod
+    def proxy_unpack(cls, args, **kw):
+        _, export_id, dict_name, key = args
+        obj = cls(export_id, dict_name=dict_name, key=key, **kw)
+        return obj
+
 
     @classmethod
     def new_by_local(cls, export, addr):
@@ -1118,7 +1141,10 @@ def get_addr(export_or_proxy):
         return svr.addr
 
 
-
+PROXY_DICT_ITEM = 1
+PROXYS = {
+    PROXY_DICT_ITEM: DictItemProxy,
+}
 
 
 
